package com.ruoyi.project.activiti.controller;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.config.activiti.ProcessDiagramGenarateExt;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.framework.web.page.TableDataInfo;
import com.ruoyi.project.activiti.domain.*;
import com.ruoyi.project.activiti.service.LeaveNewService;
import com.ruoyi.project.activiti.service.impl.MyProcessInstanceHighlightsResource;
import com.ruoyi.project.system.domain.SysUser;
import org.activiti.bpmn.model.*;
import org.activiti.bpmn.model.Process;
import org.activiti.engine.*;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.history.HistoricProcessInstance;
import org.activiti.engine.history.HistoricTaskInstance;
import org.activiti.engine.impl.RepositoryServiceImpl;
import org.activiti.engine.impl.persistence.entity.ProcessDefinitionEntity;
import org.activiti.engine.impl.pvm.process.ActivityImpl;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Array;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 请假业务Controller
 *
 * @author Xianlu Tech
 * @date 2019-10-11
 */
@RestController
@RequestMapping("/leaveNew")
public class LeaveNewController extends BaseController {
    @Autowired
    private LeaveNewService leaveNewService;

    @Autowired
    private TaskService taskService;

    /**
     * 更具流程key获取开始表单key
     *
     * @param processKey
     * @return
     */
    @GetMapping("/processKey/{processKey}")
    @ResponseBody
    public AjaxResult getFormData(@PathVariable("processKey") String processKey) {
        return AjaxResult.success(leaveNewService.getFormData(processKey));
    }

    @PutMapping
    @ResponseBody
    public AjaxResult add(@RequestBody MyFormDataActiviti myFormDataActiviti) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Long userId = user.getUserId();
        if (SysUser.isAdmin(userId)) {
            return AjaxResult.error("提交申请失败：不允许管理员提交申请！");
        }
        ;
        myFormDataActiviti.setApplyUser(userId.toString());
        myFormDataActiviti.setCreateUser(user.getUserName());
        myFormDataActiviti.setCreateDept(user.getDept().getDeptName());
        myFormDataActiviti.setCreateBy(userId.toString());
        myFormDataActiviti.setCreateTime(new Date());
        return toAjax(leaveNewService.add(myFormDataActiviti));
    }


    @GetMapping("/list")
    @ResponseBody
    public TableDataInfo list() {
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        startPage();
        List<MyFormDataActiviti> list = leaveNewService.select(userId);
        return getDataTable(list);
    }

    @GetMapping("/task")
    @ResponseBody
    public AjaxResult task(String id) {
        List<MyTaskVo> list = leaveNewService.taskList(id);
        return AjaxResult.success(list);
    }

    @GetMapping("/{id}")
    @ResponseBody
    public AjaxResult get(@PathVariable("id") Long id) {
        MyFormDataActiviti myFormDataActiviti = leaveNewService.get(id);
        return AjaxResult.success(myFormDataActiviti);
    }

    @GetMapping("/showForms/{instanceId}")
    @ResponseBody
    public AjaxResult showForms(@PathVariable("instanceId") String instanceId) {
        List<FormsVo> list = leaveNewService.showForms(instanceId);
        return AjaxResult.success(list);
    }

    @PostMapping
    @ResponseBody
    public AjaxResult update(@RequestBody MyFormDataActiviti myFormDataActiviti) {
        return toAjax(leaveNewService.update(myFormDataActiviti));
    }

    /**
     * 删除请假业务
     */

    @DeleteMapping
    @ResponseBody
    public AjaxResult remove(@RequestBody List<Long> ids) {
        return toAjax(leaveNewService.remove(ids));
    }

    /**
     * 提交申请
     *
     * @param id
     * @return
     */
    @PostMapping("/submitApply/{id}")
    @ResponseBody
    public AjaxResult submitApply(@PathVariable("id") Long id) {
        leaveNewService.submitApply(id);
        return AjaxResult.success();
    }

    @Autowired
    private RepositoryService repositoryService;

    @RequestMapping(value = "/read-resource")
    public void readResource(@RequestParam("processDefinitionKey") String processDefinitionKey, @RequestParam("version") Integer version, @RequestParam("processInstanceId") String processInstanceId, HttpServletResponse response)
            throws Exception {
        // 设置页面不缓存
        showImage(processDefinitionKey, version, processInstanceId, response);
    }

    @RequestMapping(value = "/read-activiti")
    public void readActiviti(@RequestParam("processDefinitionKey") String processDefinitionKey, @RequestParam("version") Integer version, HttpServletResponse response)
            throws Exception {
        // 设置页面不缓存
        response.setHeader("Pragma", "No-cache");
        response.setHeader("Cache-Control", "no-cache");
        response.setDateHeader("Expires", 0);

        ProcessDefinitionQuery pdq = repositoryService.createProcessDefinitionQuery();

        ProcessDefinition pd = pdq.processDefinitionKey(processDefinitionKey).processDefinitionVersion(version).singleResult();

        BpmnModel bpmnModel = repositoryService.getBpmnModel(pd.getId());

        // 获取流程图图像字符流
        //ProcessDiagramGenerator pec = processEngine.getProcessEngineConfiguration().getProcessDiagramGenerator();
        // 获取流程图图像字符流
        ProcessDiagramGenarateExt pec = new ProcessDiagramGenarateExt();
        //配置字体
        InputStream imageStream = pec.generateDiagram(bpmnModel, "png", "宋体", "微软雅黑", "黑体", null, 2.0);

        response.setContentType("image/png");
        OutputStream os = response.getOutputStream();
        int bytesRead = 0;
        byte[] buffer = new byte[8192];
        while ((bytesRead = imageStream.read(buffer, 0, 8192)) != -1) {
            os.write(buffer, 0, bytesRead);
        }
        os.close();
        imageStream.close();
    }

    /**
     * 查询代办事项
     *
     * @return
     */
    @GetMapping("/taskList")
    @ResponseBody
    public TableDataInfo taskList() {
        TableDataInfo tableDataInfo = leaveNewService.findTodoTasks();
        return tableDataInfo;
    }

    /**
     * 更具流程key获取开始表单key
     *
     * @param processKey
     * @return
     */
    @GetMapping("/taskFormData")
    @ResponseBody
    public AjaxResult taskFormData(MyTaskVo myTaskVo) {
        MyFormData myFormData = leaveNewService.taskFormData(myTaskVo);
        return AjaxResult.success(myFormData);
    }

    /**
     * 完成task
     *
     * @return
     */
    @PostMapping("/taskCommit")
    @ResponseBody
    public AjaxResult taskCommit(@RequestBody TaskCommitForm taskCommitForm) {
        leaveNewService.taskCommit(taskCommitForm);
        return AjaxResult.success();
    }


    @GetMapping("/taskDoneList")
    @ResponseBody
    public TableDataInfo taskDoneList() {
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        startPage();
        List<MyFormDataActiviti> list = leaveNewService.taskDoneList(userId);
        return getDataTable(list);
    }

    @Autowired
    private HistoryService historyService;

    @Autowired
    private ProcessEngine processEngine;

    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private MyProcessInstanceHighlightsResource myProcessInstanceHighlightsResource;

    private void showImage(String processDefinitionKey, Integer version, String processInstanceId, HttpServletResponse response) throws IOException {
        //logger.info("[开始]-获取流程图图像");
        try {
            //  获取历史流程实例
            HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery()
                    .processInstanceId(processInstanceId).singleResult();

            if (historicProcessInstance == null) {
                //throw new BusinessException("获取流程实例ID[" + processInstanceId + "]对应的历史流程实例失败！");
            } else {
                // 获取流程定义
                ProcessDefinitionEntity processDefinition = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService)
                        .getDeployedProcessDefinition(historicProcessInstance.getProcessDefinitionId());

                // 获取流程历史中已执行节点，并按照节点在流程中执行先后顺序排序
                List<HistoricActivityInstance> historicActivityInstanceList = historyService.createHistoricActivityInstanceQuery()
                        .processInstanceId(processInstanceId).orderByHistoricActivityInstanceId().asc().list();
                // 已执行的节点ID集合
                List<String> executedActivityIdList = new ArrayList<String>();
                //logger.info("获取已经执行的节点ID");
                for (HistoricActivityInstance activityInstance : historicActivityInstanceList) {
                    executedActivityIdList.add(activityInstance.getActivityId());
                }
                BpmnModel bpmnModel = repositoryService.getBpmnModel(historicProcessInstance.getProcessDefinitionId());

                // 已执行的线集合
                List<String> flowIds = new ArrayList<>();
                // 获取流程走过的线 (getHighLightedFlows是下面的方法)
                flowIds = myProcessInstanceHighlightsResource.getHighLightedFlows(processDefinition, processInstanceId);

                List<Task> listTask = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
                //当前节点信息
                List<String> nowActivityIdList = new ArrayList<String>();
                for (Task task : listTask) {
                    String taskDefinitionKey = task.getTaskDefinitionKey();
                    nowActivityIdList.add(taskDefinitionKey);
                }
                // 获取流程图图像字符流
                ProcessDiagramGenarateExt pec = new ProcessDiagramGenarateExt();

                ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(processInstanceId).singleResult();
                boolean end = false;
                if (StringUtils.isNull(processInstance)) {
                    end = true;
                }
                //配置字体
                InputStream imageStream = pec.generateDiagram(end, bpmnModel, "png",nowActivityIdList, executedActivityIdList, flowIds, "宋体", "宋体", "宋体", null, 2.0);
//                InputStream inputStream = pec.generateDiagram(bpmnModel, "png", executedActivityIdList);

                response.setContentType("image/png");
                OutputStream os = response.getOutputStream();
                int bytesRead = 0;
                byte[] buffer = new byte[8192];
                while ((bytesRead = imageStream.read(buffer, 0, 8192)) != -1) {
                    os.write(buffer, 0, bytesRead);
                }
                os.close();
                imageStream.close();
            }
            //logger.info("[完成]-获取流程图图像");
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("【异常】-获取流程图失败！" + e.getMessage());
            throw new RuntimeException("获取流程图失败！" + e.getMessage());
        }
    }



    private static List<String> executedFlowIdList(BpmnModel bpmnModel, ProcessDefinitionEntity processDefinitionEntity,
                                                   List<HistoricActivityInstance> historicActivityInstanceList) {

        List<String> executedFlowIdList = new ArrayList<>();
        Set<String> ids = new HashSet<>();
        Map<String, SequenceFlow> flowHashMap = new HashMap<>();
        List<String> activityIds = historicActivityInstanceList.stream().map(X -> X.getActivityId()).collect(Collectors.toList());

        for (int i = 0; i < historicActivityInstanceList.size(); i++) {
            HistoricActivityInstance hai = historicActivityInstanceList.get(i);
            String activityId = hai.getActivityId();
            FlowNode flowNode = (FlowNode) bpmnModel.getFlowElement(activityId);
            List<SequenceFlow> sequenceFlows = flowNode.getOutgoingFlows();
            for (SequenceFlow sequenceFlow : sequenceFlows) {
                String sourceRef = sequenceFlow.getSourceRef();
                String targetRef = sequenceFlow.getTargetRef();
                flowHashMap.put(targetRef, sequenceFlow);
            }
        }
        for (String activityId : activityIds) {
            for (String s : flowHashMap.keySet()) {
                if (s.contains(activityId)) {
                    SequenceFlow sequenceFlow = flowHashMap.get(s);
                    ids.add(sequenceFlow.getId());
                }
            }
        }
        executedFlowIdList.addAll(ids);
        return executedFlowIdList;
    }


    /**
     * 获取流程图像，已执行节点和流程线高亮显示
     */
    public void getActivitiProccessImage(String processInstanceId, HttpServletResponse response) {
        //logger.info("[开始]-获取流程图图像");
        try {
            //  获取历史流程实例
            HistoricProcessInstance historicProcessInstance = historyService.createHistoricProcessInstanceQuery()
                    .processInstanceId(processInstanceId).singleResult();

            if (historicProcessInstance == null) {
                //throw new BusinessException("获取流程实例ID[" + processInstanceId + "]对应的历史流程实例失败！");
            } else {
                // 获取流程定义
                ProcessDefinitionEntity processDefinition = (ProcessDefinitionEntity) ((RepositoryServiceImpl) repositoryService)
                        .getDeployedProcessDefinition(historicProcessInstance.getProcessDefinitionId());

                // 获取流程历史中已执行节点，并按照节点在流程中执行先后顺序排序
                List<HistoricActivityInstance> historicActivityInstanceList = historyService.createHistoricActivityInstanceQuery()
                        .processInstanceId(processInstanceId).orderByHistoricActivityInstanceId().asc().list();

                // 已执行的节点ID集合
                List<String> executedActivityIdList = new ArrayList<String>();
                int index = 1;
                //logger.info("获取已经执行的节点ID");
                for (HistoricActivityInstance activityInstance : historicActivityInstanceList) {
                    executedActivityIdList.add(activityInstance.getActivityId());

                    //logger.info("第[" + index + "]个已执行节点=" + activityInstance.getActivityId() + " : " +activityInstance.getActivityName());
                    index++;
                }

                BpmnModel bpmnModel = repositoryService.getBpmnModel(historicProcessInstance.getProcessDefinitionId());

                // 已执行的线集合
                List<String> flowIds = new ArrayList<String>();
                // 获取流程走过的线 (getHighLightedFlows是下面的方法)
                flowIds = getHighLightedFlows(bpmnModel, processDefinition, historicActivityInstanceList);

                // 获取流程图图像字符流
                //ProcessDiagramGenerator pec = processEngine.getProcessEngineConfiguration().getProcessDiagramGenerator();
                ProcessDiagramGenarateExt pec = new ProcessDiagramGenarateExt();
                //配置字体
                InputStream imageStream = pec.generateDiagram(bpmnModel, "png", executedActivityIdList, flowIds, "宋体", "微软雅黑", "黑体", null, 2.0);

                response.setContentType("image/png");
                OutputStream os = response.getOutputStream();
                int bytesRead = 0;
                byte[] buffer = new byte[8192];
                while ((bytesRead = imageStream.read(buffer, 0, 8192)) != -1) {
                    os.write(buffer, 0, bytesRead);
                }
                os.close();
                imageStream.close();
            }
            //logger.info("[完成]-获取流程图图像");
        } catch (Exception e) {
            System.out.println(e.getMessage());
            //logger.error("【异常】-获取流程图失败！" + e.getMessage());
            //throw new BusinessException("获取流程图失败！" + e.getMessage());
        }
    }

    public List<String> getHighLightedFlows(BpmnModel bpmnModel, ProcessDefinitionEntity processDefinitionEntity, List<HistoricActivityInstance> historicActivityInstances) {
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"); //24小时制
        List<String> highFlows = new ArrayList<String>();// 用以保存高亮的线flowId

        for (int i = 0; i < historicActivityInstances.size() - 1; i++) {
            // 对历史流程节点进行遍历
            // 得到节点定义的详细信息
            FlowNode activityImpl = (FlowNode) bpmnModel.getMainProcess().getFlowElement(historicActivityInstances.get(i).getActivityId());


            List<FlowNode> sameStartTimeNodes = new ArrayList<FlowNode>();// 用以保存后续开始时间相同的节点
            FlowNode sameActivityImpl1 = null;

            HistoricActivityInstance activityImpl_ = historicActivityInstances.get(i);// 第一个节点
            HistoricActivityInstance activityImp2_;

            for (int k = i + 1; k <= historicActivityInstances.size() - 1; k++) {
                activityImp2_ = historicActivityInstances.get(k);// 后续第1个节点

                if (activityImpl_.getActivityType().equals("userTask") && activityImp2_.getActivityType().equals("userTask") &&
                        df.format(activityImpl_.getStartTime()).equals(df.format(activityImp2_.getStartTime()))) //都是usertask，且主节点与后续节点的开始时间相同，说明不是真实的后继节点
                {

                } else {
                    sameActivityImpl1 = (FlowNode) bpmnModel.getMainProcess().getFlowElement(historicActivityInstances.get(k).getActivityId());//找到紧跟在后面的一个节点
                    break;
                }

            }
            sameStartTimeNodes.add(sameActivityImpl1); // 将后面第一个节点放在时间相同节点的集合里
            for (int j = i + 1; j < historicActivityInstances.size() - 1; j++) {
                HistoricActivityInstance activityImpl1 = historicActivityInstances.get(j);// 后续第一个节点
                HistoricActivityInstance activityImpl2 = historicActivityInstances.get(j + 1);// 后续第二个节点

                if (df.format(activityImpl1.getStartTime()).equals(df.format(activityImpl2.getStartTime()))) {// 如果第一个节点和第二个节点开始时间相同保存
                    FlowNode sameActivityImpl2 = (FlowNode) bpmnModel.getMainProcess().getFlowElement(activityImpl2.getActivityId());
                    sameStartTimeNodes.add(sameActivityImpl2);
                } else {// 有不相同跳出循环
                    break;
                }
            }
            List<SequenceFlow> pvmTransitions = activityImpl.getOutgoingFlows(); // 取出节点的所有出去的线

            for (SequenceFlow pvmTransition : pvmTransitions) {// 对所有的线进行遍历
                FlowNode pvmActivityImpl = (FlowNode) bpmnModel.getMainProcess().getFlowElement(pvmTransition.getTargetRef());// 如果取出的线的目标节点存在时间相同的节点里，保存该线的id，进行高亮显示
                if (sameStartTimeNodes.contains(pvmActivityImpl)) {
                    highFlows.add(pvmTransition.getId());
                }
            }

        }
        return highFlows;

    }

}
